package dev.shadowsoffire.apotheosis.adventure.affix.effect;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.Apoth;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixInstance;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixType;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.placebo.util.PlaceboUtil;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.function.Function;
import net.minecraft.ChatFormatting;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.core.Vec3i;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.util.Mth;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.ai.attributes.Attribute;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.ClipContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.phys.Vec3;
import net.minecraftforge.common.ForgeMod;
import net.minecraftforge.event.level.BlockEvent;

public class RadialAffix extends Affix {
  public static final Codec<dev.shadowsoffire.apotheosis.adventure.affix.effect.RadialAffix> CODEC;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group((App)LootRarity.mapCodec(Codec.list(RadialData.CODEC)).fieldOf("values").forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.affix.effect.RadialAffix::new));
  }
  
  private static Set<UUID> breakers = new HashSet<>();
  
  protected final Map<LootRarity, List<RadialData>> values;
  
  public RadialAffix(Map<LootRarity, List<RadialData>> values) {
    super(AffixType.ABILITY);
    this.values = values;
  }
  
  public boolean canApplyTo(ItemStack stack, LootCategory cat, LootRarity rarity) {
    return (cat.isBreaker() && this.values.containsKey(rarity));
  }
  
  public MutableComponent getDescription(ItemStack stack, LootRarity rarity, float level) {
    RadialData data = getTrueLevel(rarity, level);
    return Component.m_237110_("affix." + getId() + ".desc", new Object[] { Integer.valueOf(data.x), Integer.valueOf(data.y) });
  }
  
  public Component getAugmentingText(ItemStack stack, LootRarity rarity, float level) {
    MutableComponent comp = getDescription(stack, rarity, level);
    RadialData min = getTrueLevel(rarity, 0.0F);
    RadialData max = getTrueLevel(rarity, 1.0F);
    if (min != max) {
      MutableComponent mutableComponent1 = Component.m_237110_("%sx%s", new Object[] { Integer.valueOf(min.x), Integer.valueOf(min.y) });
      MutableComponent mutableComponent2 = Component.m_237110_("%sx%s", new Object[] { Integer.valueOf(max.x), Integer.valueOf(max.y) });
      comp.m_7220_((Component)valueBounds((Component)mutableComponent1, (Component)mutableComponent2));
    } 
    return (Component)comp;
  }
  
  public void onBreak(BlockEvent.BreakEvent e) {
    Player player = e.getPlayer();
    ItemStack tool = player.m_21205_();
    Level world = player.m_9236_();
    if (!world.f_46443_ && tool.m_41782_()) {
      AffixInstance inst = (AffixInstance)AffixHelper.getAffixes(tool).get(Apoth.Affixes.RADIAL);
      if (inst != null && inst.isValid() && RadialState.getState(player).isRadialMiningEnabled(player)) {
        float hardness = e.getState().m_60800_((BlockGetter)e.getLevel(), e.getPos());
        breakExtraBlocks((ServerPlayer)player, e.getPos(), tool, getTrueLevel((LootRarity)inst.rarity().get(), inst.level()), hardness);
      } 
    } 
  }
  
  private RadialData getTrueLevel(LootRarity rarity, float level) {
    List<RadialData> list = this.values.get(rarity);
    return list.get(Math.min(list.size() - 1, (int)Mth.m_14179_(level, 0.0F, list.size())));
  }
  
  public Codec<? extends Affix> getCodec() {
    return (Codec)CODEC;
  }
  
  public static void toggleRadialState(Player player) {
    RadialState state = RadialState.getState(player);
    RadialState next = state.next();
    RadialState.setState(player, next);
    player.m_213846_((Component)Apotheosis.sysMessageHeader().m_7220_((Component)Component.m_237110_("misc.apotheosis.radial_state_updated", new Object[] { next.toComponent(), state.toComponent() }).m_130940_(ChatFormatting.YELLOW)));
  }
  
  public static void breakExtraBlocks(ServerPlayer player, BlockPos pos, ItemStack tool, RadialData level, float hardness) {
    if (!breakers.add(player.m_20148_()))
      return; 
    try {
      breakBlockRadius(player, pos, level.x, level.y, level.xOff, level.yOff, hardness);
    } catch (Exception e) {
      e.printStackTrace();
    } 
    breakers.remove(player.m_20148_());
  }
  
  public static void breakBlockRadius(ServerPlayer player, BlockPos pos, int x, int y, int xOff, int yOff, float hardness) {
    Level world = player.m_9236_();
    if (x < 2 && y < 2)
      return; 
    int lowerY = (int)Math.ceil(-y / 2.0D), upperY = (int)Math.round(y / 2.0D);
    int lowerX = (int)Math.ceil(-x / 2.0D), upperX = (int)Math.round(x / 2.0D);
    Vec3 base = player.m_20299_(0.0F);
    Vec3 look = player.m_20154_();
    double reach = player.m_21133_((Attribute)ForgeMod.BLOCK_REACH.get());
    Vec3 target = base.m_82520_(look.f_82479_ * reach, look.f_82480_ * reach, look.f_82481_ * reach);
    BlockHitResult blockHitResult1 = world.m_45547_(new ClipContext(base, target, ClipContext.Block.OUTLINE, ClipContext.Fluid.NONE, (Entity)player));
    if (blockHitResult1 == null || blockHitResult1.m_6662_() != HitResult.Type.BLOCK)
      return; 
    BlockHitResult res = blockHitResult1;
    Direction face = res.m_82434_();
    for (int iy = lowerY; iy < upperY; iy++) {
      for (int ix = lowerX; ix < upperX; ix++) {
        BlockPos genPos = new BlockPos(pos.m_123341_() + ix + xOff, pos.m_123342_() + iy + yOff, pos.m_123343_());
        if (player.m_6350_().m_122434_() == Direction.Axis.X)
          genPos = new BlockPos(genPos.m_123341_() - ix + xOff, genPos.m_123342_(), genPos.m_123343_() + ix + xOff); 
        if (face.m_122434_().m_122478_())
          genPos = rotateDown(genPos, iy + yOff, player.m_6350_()); 
        if (!genPos.equals(pos)) {
          BlockState state = world.m_8055_(genPos);
          float stateHardness = state.m_60800_((BlockGetter)world, genPos);
          if (!state.m_60795_() && stateHardness != -1.0F && stateHardness <= hardness * 3.0F && isEffective(state, (Player)player))
            PlaceboUtil.tryHarvestBlock(player, genPos); 
        } 
      } 
    } 
  }
  
  static BlockPos rotateDown(BlockPos pos, int y, Direction horizontal) {
    Vec3i vec = horizontal.m_122436_();
    return new BlockPos(pos.m_123341_() + vec.m_123341_() * y, pos.m_123342_() - y, pos.m_123343_() + vec.m_123343_() * y);
  }
  
  static boolean isEffective(BlockState state, Player player) {
    return player.m_36298_(state);
  }
}
